import React from 'react';

const useClickOutside = (ref, callback) => {
  const handlerRef = React.useRef(callback);

  /**
   * Update callback if it changes
   */
  React.useEffect(() => {
    handlerRef.current = callback;
  });

  /**
   * Add and remove event listeners
   */
  React.useEffect(() => {
    const listener = event => {
      if (!ref || !ref.current) {
        return;
      }

      /**
       * Handle click outside of shadow DOM
       */
      if (event.target.shadowRoot !== null) {
        if (!event.target.shadowRoot.contains(ref.current)) {
          handlerRef.current(event);
        }
      } else {
        if (!event.target.contains(ref.current)) {
          handlerRef.current(event);
        }
      }
    };

    document.addEventListener('click', listener);
    document.addEventListener('touch', listener);

    return () => {
      document.removeEventListener('click', listener);
      document.removeEventListener('touch', listener);
    };
  });
};

export default useClickOutside;
